from urllib import request


class BaseMonitoringTest():
    response = None

    def check(self):
        return True


# Django tests
class BaseDataBaseMonitoringTest(BaseMonitoringTest):
    cls = None

    def __init__(self, cls):
        self.cls = cls

    def check(self):
        try:
            return len(self.cls.objects.all()) >= 0
        except Exception as e:
            self.response = f'BaseDataBaseMonitoringTest Error {str(e)}'
            return False


# AWS internal services tests
class BaseElasticacheMonitoringTest(BaseMonitoringTest):
    django_rq = None

    def __init__(self, django_rq):
        self.django_rq = django_rq

    def check(self):
        try:
            self.django_rq.get_connection('default')
            return True
        except Exception as e:
            self.response = f'BaseElasticacheMonitoringTest Error {str(e)}'
            return False


class BaseKinesisMonitoringTest(BaseMonitoringTest):
    client = None
    stream_name = None

    def __init__(self, client, stream_name):
        self.client = client
        self.stream_name = stream_name

    def check(self):
        try:
            self.client.list_shards(
                StreamName=self.stream_name,
                MaxResults=1
            )
            return True
        except Exception as e:
            self.response = f'BaseKinesisMonitoringTest Error {str(e)}'
            return False


class BaseSQSMonitoringTest(BaseMonitoringTest):
    client = None
    queue_name = None

    def __init__(self, client, queue_name):
        self.client = client
        self.queue_name = queue_name

    def check(self):
        try:
            self.client.get_queue_by_name(self.queue_name)
            return True
        except Exception as e:
            self.response = f'BaseSQSMonitoringTest Error {str(e)}'
            return False


class BaseS3MonitoringTest(BaseMonitoringTest):
    client = None
    bucket_name = None

    def __init__(self, client, bucket_name):
        self.client = client
        self.bucket_name = bucket_name

    def check(self):
        try:
            self.client.Bucket(self.bucket_name)
            return True
        except Exception as e:
            self.response = f'BaseS3MonitoringTest Error {str(e)}'
            return False


# External services tests
class BaseAlgoliaMonitoringTest(BaseMonitoringTest):
    index = None

    def __init__(self, index):
        self.index = index

    def check(self):
        try:
            self.index.search('')
            return True
        except Exception as e:
            self.response = f'BaseAlgoliaMonitoringTest Error {str(e)}'
            return False


class BaseFirebaseMonitoringTest(BaseMonitoringTest):
    client = None

    def __init__(self, client):
        self.client = client

    def check(self):
        try:
            self.client.list_users(max_results=1)
            return True
        except Exception as e:
            self.response = f'BaseFirebaseMonitoringTest Error {str(e)}'
            return False


class BaseExternalUrlMonitoringTest(BaseMonitoringTest):
    url = None

    def __init__(self, url):
        self.url = url

    def check(self):
        try:
            status_code = request.urlopen(self.url).getcode()
            if status_code <= 299:
                return True
            self.response = 'BaseExternalUrlMonitoringTest Error: ' \
                            + f'Status code {status_code}'
            return False
        except Exception as e:
            self.response = f'BaseExternalUrlMonitoringTest Error {str(e)}'
            return False



from rest_framework import status, viewsets
from rest_framework.decorators import action
from rest_framework.response import Response

from django.http import HttpResponse
from django.views import View


class BaseMonitoringView(View):
    tests = []

    def get(self, request):
        failed = []

        for test in self.tests:
            if not test.check():
                failed.append(test.response)

        if len(failed) > 0:
            return HttpResponse(failed, status.HTTP_500_INTERNAL_SERVER_ERROR)

        return HttpResponse('OK', status.HTTP_200_OK)


class BaseMonitoringViewSet(viewsets.GenericViewSet):
    ping_tests = []
    pong_tests = []

    @action(
        detail=False,
        methods=['get'],
        url_path=''
    )
    def ping(self, request):
        failed = []

        for test in self.ping_tests:
            if not test.check():
                failed.append(test.response)

        if len(failed) > 0:
            return Response(
                failed,
                status=status.HTTP_500_INTERNAL_SERVER_ERROR
            )

        return Response('OK', status=status.HTTP_200_OK)

    @action(
        detail=False,
        methods=['get'],
        url_path=''
    )
    def pong(self, request):
        failed = []

        for test in self.pong_tests:
            if not test.check():
                failed.append(test.response)

        if len(failed) > 0:
            return Response(
                failed,
                status=status.HTTP_500_INTERNAL_SERVER_ERROR
            )

        return Response('OK', status=status.HTTP_200_OK)



import logging

from re import fullmatch

from django.utils.translation import activate
from django.http import HttpResponseBadRequest
from django.conf import settings

logger = logging.getLogger()


class LocaleMiddleware(object):
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        logger.debug(request.headers)

        if not any(path in request.path for path in settings.EXCLUDE_PATHS):
            accept_language = request.META.get('HTTP_ACCEPT_LANGUAGE')

            if not accept_language:
                return HttpResponseBadRequest(
                    'Add Accept-Language to the header. Example: '
                    + 'Accept-Language: pt-PT'
                )

            accept_language = accept_language.split(',')[0] if ',' \
                in accept_language else accept_language
            accept_language = accept_language.split(';')[0] if ';' \
                in accept_language else accept_language

            if not fullmatch('^[a-z]{2}-[A-Z]{2}$', accept_language):
                return HttpResponseBadRequest(
                    'Incorrect Accept-Language. Example: '
                    + 'Accept-Language: pt-PT'
                )

            # V2.13.6: Temporary fix
            if accept_language not in ['el-GR', 'en-GB', 'de-DE', 'it-IT',
                                       'fr-FR', 'es-ES', 'pt-PT']:
                logger.warning('This accept language does not exist. ' +
                               f'Accept Language: {accept_language};')
                accept_language = 'pt-PT'
            # End of logic

            activate(accept_language)

            request.locale = accept_language

        response = self.get_response(request)

        return response
